import { MessageBrokerPublisher } from '@mobicoop/message-broker-module';
import { Module, Provider } from '@nestjs/common';
import { CqrsModule } from '@nestjs/cqrs';
import {
  AD_MESSAGE_PUBLISHER,
  AD_REPOSITORY,
  INPUT_DATETIME_TRANSFORMER,
  OUTPUT_DATETIME_TRANSFORMER,
  TIMEZONE_FINDER,
  TIME_CONVERTER,
} from './ad.di-tokens';
import { AdMapper } from './ad.mapper';
import { CreateAdService } from './core/application/commands/create-ad/create-ad.service';
import { DeleteAdService } from './core/application/commands/delete-ad/delete-ad.service';
import { DeleteUserAdsService } from './core/application/commands/delete-user-ads/delete-user-ads.service';
import { InvalidateAdService } from './core/application/commands/invalidate-ad/invalidate-ad.service';
import { UpdateAdService } from './core/application/commands/update-ad/update-ad.service';
import { ValidateAdService } from './core/application/commands/validate-ad/validate-ad.service';
import { PublishMessageWhenAdIsDeletedDomainEventHandler } from './core/application/event-handlers/publish-message-when-ad-deleted.domain-event-handler';
import { PublishMessageWhenAdIsCreatedDomainEventHandler } from './core/application/event-handlers/publish-message-when-ad-is-created.domain-event-handler';
import { PublishMessageWhenAdIsUpdatedDomainEventHandler } from './core/application/event-handlers/publish-message-when-ad-is-updated.domain-event-handler';
import { FindAdByIdQueryHandler } from './core/application/queries/find-ad-by-id/find-ad-by-id.query-handler';
import { FindAdsByIdsQueryHandler } from './core/application/queries/find-ads-by-ids/find-ads-by-ids.query-handler';
import { FindAdsByUserIdQueryHandler } from './core/application/queries/find-ads-by-user-id/find-ads-by-user-id.query-handler';
import { AdRepository } from './infrastructure/ad.repository';
import { InputDateTimeTransformer } from './infrastructure/input-datetime-transformer';
import { OutputDateTimeTransformer } from './infrastructure/output-datetime-transformer';
import { PrismaService } from './infrastructure/prisma.service';
import { TimeConverter } from './infrastructure/time-converter';
import { TimezoneFinder } from './infrastructure/timezone-finder';
import { CreateAdGrpcController } from './interface/grpc-controllers/create-ad.grpc.controller';
import { DeleteAdGrpcController } from './interface/grpc-controllers/delete-ad.grpc.controller';
import { FindAdByIdGrpcController } from './interface/grpc-controllers/find-ad-by-id.grpc.controller';
import { FindAdsByIdsGrpcController } from './interface/grpc-controllers/find-ads-by-ids.grpc.controller';
import { FindAdsByUserIdGrpcController } from './interface/grpc-controllers/find-ads-by-user-id.grpc.controller';
import { MatcherAdCreatedMessageHandler } from './interface/message-handlers/matcher-ad-created.message-handler';
import { MatcherAdCreationFailedMessageHandler } from './interface/message-handlers/matcher-ad-creation-failed.message-handler';
import { UserDeletedMessageHandler } from './interface/message-handlers/user-deleted.message-handler';

const grpcControllers = [
  CreateAdGrpcController,
  DeleteAdGrpcController,
  FindAdByIdGrpcController,
  FindAdsByIdsGrpcController,
  FindAdsByUserIdGrpcController,
];

const messageHandlers = [
  MatcherAdCreatedMessageHandler,
  MatcherAdCreationFailedMessageHandler,
  UserDeletedMessageHandler,
];

const eventHandlers: Provider[] = [
  PublishMessageWhenAdIsCreatedDomainEventHandler,
  PublishMessageWhenAdIsUpdatedDomainEventHandler,
  PublishMessageWhenAdIsDeletedDomainEventHandler,
];

const commandHandlers: Provider[] = [
  CreateAdService,
  UpdateAdService,
  DeleteAdService,
  DeleteUserAdsService,
  ValidateAdService,
  InvalidateAdService,
];

const queryHandlers: Provider[] = [
  FindAdByIdQueryHandler,
  FindAdsByIdsQueryHandler,
  FindAdsByUserIdQueryHandler,
];

const mappers: Provider[] = [AdMapper];

const repositories: Provider[] = [
  {
    provide: AD_REPOSITORY,
    useClass: AdRepository,
  },
];

const messagePublishers: Provider[] = [
  {
    provide: AD_MESSAGE_PUBLISHER,
    useExisting: MessageBrokerPublisher,
  },
];
const orms: Provider[] = [PrismaService];

const adapters: Provider[] = [
  {
    provide: TIMEZONE_FINDER,
    useClass: TimezoneFinder,
  },
  {
    provide: TIME_CONVERTER,
    useClass: TimeConverter,
  },
  {
    provide: INPUT_DATETIME_TRANSFORMER,
    useClass: InputDateTimeTransformer,
  },
  {
    provide: OUTPUT_DATETIME_TRANSFORMER,
    useClass: OutputDateTimeTransformer,
  },
];

@Module({
  imports: [CqrsModule],
  controllers: [...grpcControllers],
  providers: [
    ...messageHandlers,
    ...eventHandlers,
    ...commandHandlers,
    ...queryHandlers,
    ...mappers,
    ...repositories,
    ...messagePublishers,
    ...orms,
    ...adapters,
  ],
  exports: [PrismaService, AdMapper, AD_REPOSITORY, TIMEZONE_FINDER],
})
export class AdModule {}
