import { AggregateID, ConflictException } from '@mobicoop/ddd-library';
import {
  AD_REPOSITORY,
  INPUT_DATETIME_TRANSFORMER,
} from '@modules/ad/ad.di-tokens';
import { AdEntity } from '@modules/ad/core/domain/ad.entity';
import { AdAlreadyExistsException } from '@modules/ad/core/domain/ad.errors';
import { ScheduleItemProps } from '@modules/ad/core/domain/value-objects/schedule-item.value-object';
import { Inject } from '@nestjs/common';
import { CommandHandler, ICommandHandler } from '@nestjs/cqrs';
import { AdRepositoryPort } from '../../ports/ad.repository.port';
import { DateTimeTransformerPort } from '../../ports/datetime-transformer.port';
import { Waypoint } from '../../types/waypoint';
import { CreateAdCommand } from './create-ad.command';

export function createPropsFromCommand(
  command: CreateAdCommand,
  datetimeTransformer: DateTimeTransformerPort,
) {
  return {
    userId: command.userId,
    driver: command.driver,
    passenger: command.passenger,
    frequency: command.frequency,
    //TODO Shouldn't that kind of logic be in the domain layer?
    fromDate: datetimeTransformer.fromDate(
      {
        date: command.fromDate,
        time: command.schedule[0].time,
        coordinates: {
          lon: command.waypoints[0].lon,
          lat: command.waypoints[0].lat,
        },
      },
      command.frequency,
    ),
    toDate: datetimeTransformer.toDate(
      command.toDate,
      {
        date: command.fromDate,
        time: command.schedule[0].time,
        coordinates: {
          lon: command.waypoints[0].lon,
          lat: command.waypoints[0].lat,
        },
      },
      command.frequency,
    ),
    schedule: command.schedule.map((scheduleItem: ScheduleItemProps) => ({
      day: datetimeTransformer.day(
        scheduleItem.day,
        {
          date: command.fromDate,
          time: scheduleItem.time,
          coordinates: {
            lon: command.waypoints[0].lon,
            lat: command.waypoints[0].lat,
          },
        },
        command.frequency,
      ),
      time: datetimeTransformer.time(
        {
          date: command.fromDate,
          time: scheduleItem.time,
          coordinates: {
            lon: command.waypoints[0].lon,
            lat: command.waypoints[0].lat,
          },
        },
        command.frequency,
      ),
      margin: scheduleItem.margin,
    })),
    seatsProposed: command.seatsProposed ?? 0,
    seatsRequested: command.seatsRequested ?? 0,
    strict: command.strict,
    waypoints: command.waypoints.map((waypoint: Waypoint) => ({
      position: waypoint.position,
      address: {
        name: waypoint.name,
        houseNumber: waypoint.houseNumber,
        street: waypoint.street,
        postalCode: waypoint.postalCode,
        locality: waypoint.locality,
        country: waypoint.country,
        coordinates: {
          lon: waypoint.lon,
          lat: waypoint.lat,
        },
      },
    })),
    comment: command.comment,
  };
}

@CommandHandler(CreateAdCommand)
export class CreateAdService implements ICommandHandler {
  constructor(
    @Inject(AD_REPOSITORY)
    private readonly repository: AdRepositoryPort,
    @Inject(INPUT_DATETIME_TRANSFORMER)
    private readonly datetimeTransformer: DateTimeTransformerPort,
  ) {}

  async execute(command: CreateAdCommand): Promise<AggregateID> {
    const ad = AdEntity.create(
      createPropsFromCommand(command, this.datetimeTransformer),
    );

    try {
      await this.repository.insert(ad);
      return ad.id;
    } catch (error: any) {
      if (error instanceof ConflictException) {
        throw new AdAlreadyExistsException(error);
      }
      throw error;
    }
  }
}
